{# Returned HTML must be inline to work with SSE #}<p class="text-lg">Videobox is importing the series data. This may take a while&hellip;</p><img class="icon anim-spin" width="50" height="50" aria-hidden="true" src="/static/icon-update.svg" alt=""><div class="">{{message}}</div>