{% extends "base.html" %}

{% block title %}
<title>{{ language|lang }} • Videobox</title>
{% endblock %}

{% block content %}
<main id="main">
  <h1 class="text-xlg mb-2 font-weight-black">{{ language|lang }} language</h1>
  <p class="text-muted mb-5">Found <strong>{{series_count}}</strong> series</p>
  <div class="load-more-wrapper">
    {% include "_language-card-grid.html" %}
  </div>
</main>
{% endblock %}