{# Returned HTML must be inline to work with SSE #}<img width="50" height="50"  aria-hidden="true" class="icon" src="/static/icon-check.svg" alt=""><div class="">{{message}}</div><a class="btn" href="/">Start discovering series</a>