{% extends "base.html" %}

{% block title %}
<title>Page Not Found • Videobox</title>
{% endblock %}

{% block content %}
  <main id="main">
    <h1 class="text-xlg font-weight-black mb-2">Page Not Found</h1>
    <p class="text-lg">
      What you were looking for is just not there.
    </p>
  </main>
{% endblock %}