<div class="dialog-body">
    <form method="dialog">
        <h2 class="text-lg mb-3 text-break-all">{{release.name}}
        </h2>
        {% set languages = release.languages %}
        {% if languages %}
            <div class="d-flex p-2 mb-3 border-accent align-items-center rounded">
                <img src="/static/icon-info.svg" class="icon mr-2" alt="Information" width="20" height="20">
                <div class="text-sm">
                    It looks like this episode contains subtitles or has been dubbed in: {{languages|join(", ")}}
                </div>
            </div>
        {% endif %}
        <p class="py-3 d-flex align-items-center border-top border-bottom">
            <img class="icon mr-2" width="20" height="20" src="/static/icon-magnet.svg"><a download href="{{release.magnet_uri}}">Download torrent with magnet link</a>
        </p>
        <dl class="mb-4">
            <dt>
                Added
            </dt>
            <dd>
                {{release.added_on | human_date_time}} UTC
            </dd>
            <dt>Size</dt>
            <dd>
                {{release.size | filesizeformat}} ({{release.size}} bytes)
            </dd>
            <dt>
                Info hash
            </dt>
            <dd class="">
                <code>{{release.info_hash | upper}}</code>
            </dd>
            <dt>
                Swarm
            </dt>
            <dd>
                {{"{:,}".format(release.seeders)}} seeders, {{"{:,}".format(release.leechers)}} leechers, and {{"{:,}".format(release.completed)}} downloads.<br> 
                Last checked {{release.last_updated_on|datetime_since(utc_now)}}.
            </dd>
        </dl>
        <button class="btn" type="submit">Close</button>        
    </form>
</div>