<meta charset="utf-8" />
<meta name="viewport" content="initial-scale=1.0, user-scalable=yes, width=device-width" />
<meta name="theme-color" content="#253e52">  
<link rel="stylesheet" href="{{ url_for('static', filename='app.css') }}" />
<link rel="apple-touch-icon" href="{{ url_for('static', filename='apple-touch-icon.png') }}" />
<link rel="icon" type="image/png" sizes="32x32" href="{{ url_for('static', filename='favicon.png') }}" /> 
<link rel='mask-icon' href='{{ url_for("static", filename="favicon.svg") }}' color='#b163a8' />
<script defer src="{{ url_for('static', filename='app.js') }}"></script>
