<!DOCTYPE html>
<html>

<head>
    {% block title %}
    <title>Videobox</title>
    {% endblock %}
    {% include "_meta.html" %}
</head>

{% block body %}

<body class="d-flex flex-column align-items-center justify-content-center" style="height: 100vh; gap: 2rem">    
    <div class="d-flex flex-column align-items-center" style="gap: 1rem" id="update-dialog">
        {% include "_update-progress.html" %}
    </div>    
    <script>
        window.addEventListener("DOMContentLoaded", (event) => {
            Videobox.sync()
        })
    </script>
</body>

{% endblock %}

</html>